import { NextRequest, NextResponse } from 'next/server';

// Define interfaces for Wikidata API response
interface WikidataSearchItem {
  id: string;
  label?: string;
  description?: string;
  concepturi?: string;
  url?: string;
  title?: string;
}

interface WikidataSearchResponse {
  search?: WikidataSearchItem[];
  searchinfo?: {
    search: string;
  };
  success?: number;
}

export async function GET(request: NextRequest) {
  const { searchParams } = new URL(request.url);
  const query = searchParams.get('query');

  if (!query || typeof query !== 'string') {
    return NextResponse.json({ error: 'Query parameter is required' }, { status: 400 });
  }

  try {
    const wikidataUrl = `https://www.wikidata.org/w/api.php?action=wbsearchentities&search=${encodeURIComponent(query)}&language=en&limit=5&format=json&origin=*`;
    
    const response = await fetch(wikidataUrl, {
      headers: {
        'User-Agent': 'WikidataApp/1.0 (https://example.com/contact)'
      }
    });

    if (!response.ok) {
      throw new Error(`Wikidata API error: ${response.status}`);
    }

    const data: WikidataSearchResponse = await response.json();
    
    // Transform the response to match our WikidataSuggestion interface
    const suggestions = (data.search || []).map((item: WikidataSearchItem) => ({
      id: item.id,
      label: item.label || '',
      description: item.description || ''
    }));

    return NextResponse.json(suggestions);
  } catch (error) {
    console.error('Error fetching Wikidata suggestions:', error);
    return NextResponse.json({ error: 'Failed to fetch suggestions' }, { status: 500 });
  }
}