import React, { useState, useEffect, useCallback, useRef } from 'react';
import { useQuery, useMutation, useQueryClient } from '@tanstack/react-query';
import { Item, WikidataSuggestion, PropertyLabel } from '@/types/database';
import { EditableCell } from './EditableCell';
import { TypeaheadInput } from './TypeaheadInput';
import { PropertyInput } from './PropertyInput';
import { v4 as uuidv4 } from 'uuid';

interface ItemsListProps {
  url: string;
}

interface ItemsResponse {
  items: Item[];
  selectedProperties: string[];
}

// API functions
async function loadItemsFromDb(url: string): Promise<ItemsResponse> {
  console.log('[DEBUG] Loading items for URL:', url);
  
  const encodedUrl = encodeURIComponent(url);
  const apiUrl = `/api/urls/${encodedUrl}/items`;
  
  console.log('[DEBUG] Making request to API endpoint:', apiUrl);
  
  const response = await fetch(apiUrl);
  
  if (!response.ok) {
    const errorText = await response.text();
    console.error('[ERROR] Server error:', response.status, errorText);
    throw new Error(`Server error (${response.status}): ${errorText}`);
  }
  
  const items = await response.json();
  console.log('[DEBUG] Successfully parsed', items.length, 'items');
  
  // Get selected properties
  const propertiesResponse = await fetch(`/api/urls/${encodedUrl}/properties`);
  if (!propertiesResponse.ok) {
    throw new Error(`Failed to fetch properties: ${propertiesResponse.status}`);
  }
  
  const selectedProperties = await propertiesResponse.json();
  console.log('[DEBUG] Successfully received selected properties');
  
  // Filter items to only include selected properties
  const filteredItems = items.map((item: Item) => ({
    ...item,
    customProperties: Object.fromEntries(
      Object.entries(item.customProperties).filter(([key]) => 
        selectedProperties.includes(key)
      )
    )
  }));
  
  return { items: filteredItems, selectedProperties };
}

async function saveItemToDb(url: string, item: Item): Promise<void> {
  console.log('[FRONTEND] Saving item - ID:', item.id, 'Name:', item.name);
  
  // Validate item before saving
  if (!item.id) {
    throw new Error('Item ID is required');
  }
  
  // Ensure name and description are strings
  const sanitizedItem = {
    ...item,
    name: item.name || '',
    description: item.description || '',
    customProperties: item.customProperties || {}
  };
  
  const encodedUrl = encodeURIComponent(url);
  
  try {
    const response = await fetch(`/api/urls/${encodedUrl}/items`, {
      method: 'POST',
      headers: { 'Content-Type': 'application/json' },
      body: JSON.stringify(sanitizedItem)
    });
    
    if (!response.ok) {
      const errorText = await response.text();
      console.error('[ERROR] Failed to save item:', response.status, errorText);
      
      // Handle specific database constraint errors
      if (response.status === 400 && errorText.includes('Unique constraint failed')) {
        console.log('[INFO] Item already exists, attempting update...');
        // Try to update instead of create
        const updateResponse = await fetch(`/api/urls/${encodedUrl}/items/${item.id}`, {
          method: 'PUT',
          headers: { 'Content-Type': 'application/json' },
          body: JSON.stringify(sanitizedItem)
        });
        
        if (!updateResponse.ok) {
          const updateErrorText = await updateResponse.text();
          throw new Error(`Failed to update item (${updateResponse.status}): ${updateErrorText}`);
        }
        
        console.log('[SUCCESS] Item updated successfully');
        return;
      }
      
      throw new Error(`Failed to save item (${response.status}): ${errorText}`);
    }
    
    console.log('[SUCCESS] Item saved successfully');
  } catch (error) {
    console.error('[ERROR] Error in saveItemToDb:', error);
    throw error;
  }
}

async function deleteItemFromDb(url: string, itemId: string): Promise<void> {
  const encodedUrl = encodeURIComponent(url);
  const response = await fetch(`/api/urls/${encodedUrl}/items/${itemId}`, {
    method: 'DELETE'
  });
  
  if (!response.ok) {
    throw new Error(`Failed to delete item: ${response.status}`);
  }
}

async function deletePropertyFromDb(url: string, property: string): Promise<void> {
  const encodedUrl = encodeURIComponent(url);
  const response = await fetch(`/api/urls/${encodedUrl}/properties/${property}`, {
    method: 'DELETE'
  });
  
  if (!response.ok) {
    throw new Error(`Failed to delete property: ${response.status}`);
  }
}

async function addPropertyToDb(url: string, property: string): Promise<void> {
  const encodedUrl = encodeURIComponent(url);
  const response = await fetch(`/api/urls/${encodedUrl}/properties`, {
    method: 'POST',
    headers: { 'Content-Type': 'application/json' },
    body: JSON.stringify(property)
  });
  
  if (!response.ok) {
    throw new Error(`Failed to add property: ${response.status}`);
  }
}

// Wikidata API functions
async function fetchWikidataProperties(wikidataId: string): Promise<Record<string, string>> {
  try {
    const response = await fetch(`/api/wikidata/properties?wikidataId=${encodeURIComponent(wikidataId)}`);
    
    if (!response.ok) {
      console.error('Wikidata properties API error:', response.status);
      return {};
    }
    
    const result = await response.json();
    return result;
  } catch (error) {
    console.error('Error fetching Wikidata properties:', error);
    return {};
  }
}

async function fetchPropertyLabels(propertyIds: string[]): Promise<Record<string, string>> {
  if (propertyIds.length === 0) return {};
  
  try {
    const response = await fetch('/api/wikidata/labels', {
      method: 'POST',
      headers: { 'Content-Type': 'application/json' },
      body: JSON.stringify({ propertyIds })
    });
    
    if (!response.ok) {
      console.error('Property labels API error:', response.status);
      return {};
    }
    
    const result = await response.json();
    return result;
  } catch (error) {
    console.error('Error fetching property labels:', error);
    return {};
  }
}

async function fetchWikidataSuggestions(query: string): Promise<WikidataSuggestion[]> {
  if (!query.trim()) return [];
  
  try {
    const response = await fetch(`/api/wikidata/suggestions?query=${encodeURIComponent(query)}`);
    
    if (!response.ok) {
      console.error('Wikidata suggestions API error:', response.status);
      return [];
    }
    
    const suggestions = await response.json();
    return suggestions;
  } catch (error) {
    console.error('Error fetching Wikidata suggestions:', error);
    return [];
  }
}

export function ItemsList({ url }: ItemsListProps) {
  const queryClient = useQueryClient();
  
  // State management
  const [items, setItems] = useState<Item[]>([]);
  const [selectedProperties, setSelectedProperties] = useState<Record<string, boolean>>({});
  const [customProperties, setCustomProperties] = useState<string[]>([]);
  const [propertyLabels, setPropertyLabels] = useState<Record<string, string>>({});
  const [focusedCell, setFocusedCell] = useState<string | null>(null);
  const [focusedItemId, setFocusedItemId] = useState<string | null>(null);
  const [wikidataSuggestions, setWikidataSuggestions] = useState<Record<string, WikidataSuggestion[]>>({});
  const [propertyCache, setPropertyCache] = useState<Record<string, Record<string, string>>>({});
  
  // Refs
  const newPropertyInputRef = useRef<HTMLInputElement>(null);

  // Load items from database
  const { data: itemsData, isLoading, error } = useQuery({
    queryKey: ['items', url],
    queryFn: () => loadItemsFromDb(url),
    staleTime: 30000, // 30 seconds
  });

  // Initialize state when data loads
  useEffect(() => {
    if (itemsData) {
      const { items: loadedItems, selectedProperties: selectedProps } = itemsData;
      
      if (loadedItems.length === 0) {
        // Initialize with one empty item
        const emptyItem: Item = {
          id: uuidv4(),
          name: '',
          description: '',
          wikidataId: undefined,
          customProperties: {}
        };
        setItems([emptyItem]);
      } else {
        setItems(loadedItems);
      }
      
      // Set selected properties
      const selectedPropsMap: Record<string, boolean> = {};
      selectedProps.forEach(prop => {
        selectedPropsMap[prop] = true;
      });
      setSelectedProperties(selectedPropsMap);
      
      // Extract custom properties
      const customProps = Array.from(new Set(
        loadedItems.flatMap(item => Object.keys(item.customProperties))
      ));
      setCustomProperties(customProps);
      
      // Fetch property labels
      if (customProps.length > 0) {
        fetchPropertyLabels(customProps).then(labels => {
          setPropertyLabels(prev => ({ ...prev, ...labels }));
        });
      }
    }
  }, [itemsData]);

  // Reset focus state when URL changes
  useEffect(() => {
    setFocusedCell(null);
    setFocusedItemId(null);
  }, [url]);

  // Mutations
  const saveItemMutation = useMutation({
    mutationFn: (item: Item) => saveItemToDb(url, item),
    onSuccess: () => {
      queryClient.invalidateQueries({ queryKey: ['items', url] });
    }
  });

  const deleteItemMutation = useMutation({
    mutationFn: (itemId: string) => deleteItemFromDb(url, itemId),
    onSuccess: () => {
      queryClient.invalidateQueries({ queryKey: ['items', url] });
    }
  });

  const deletePropertyMutation = useMutation({
    mutationFn: (property: string) => deletePropertyFromDb(url, property),
    onSuccess: () => {
      queryClient.invalidateQueries({ queryKey: ['items', url] });
    }
  });

  const addPropertyMutation = useMutation({
    mutationFn: (property: string) => addPropertyToDb(url, property),
  });

  // Event handlers
  const updateItem = useCallback((index: number, field: string, value: string) => {
    setItems(prevItems => {
      const newItems = [...prevItems];
      const item = { ...newItems[index] };
      
      if (field === 'name') {
        item.name = value;
        // Fetch Wikidata suggestions only if value is not empty
        if (value.trim().length >= 2) {
          fetchWikidataSuggestions(value).then(suggestions => {
            setWikidataSuggestions(prev => ({
              ...prev,
              [`name-${index}`]: suggestions
            }));
          }).catch(error => {
            console.error('Error fetching Wikidata suggestions:', error);
            // Clear suggestions on error
            setWikidataSuggestions(prev => ({
              ...prev,
              [`name-${index}`]: []
            }));
          });
        } else {
          // Clear suggestions if value is too short
          setWikidataSuggestions(prev => ({
            ...prev,
            [`name-${index}`]: []
          }));
        }
      } else if (field === 'description') {
        item.description = value;
      } else {
        item.customProperties = { ...item.customProperties, [field]: value };
      }
      
      newItems[index] = item;
      
      // Only save if the item has some content
      const hasContent = item.name.trim() || 
                        item.description.trim() || 
                        Object.values(item.customProperties).some(prop => prop.trim());
      
      if (hasContent) {
        // Auto-save with error handling
        saveItemMutation.mutate(item, {
          onError: (error) => {
            console.error('Failed to save item:', error);
            // You could add a toast notification here
          },
          onSuccess: () => {
            // After successful save, reload items to get any synchronized properties
            // This ensures that if another item with the same name exists, 
            // its properties are updated in the UI
            queryClient.invalidateQueries({ queryKey: ['items', url] });
          }
        });
      }
      
      // Add new row if editing last row and value is not empty
      // Only add if the last item doesn't already have content and we're not already at the end
      if (index === newItems.length - 1 && value.trim()) {
        const lastItem = newItems[newItems.length - 1];
        const lastItemHasContent = lastItem.name.trim() || 
                                  lastItem.description.trim() || 
                                  Object.values(lastItem.customProperties).some(prop => prop.trim());
        
        if (lastItemHasContent) {
          const newItem: Item = {
            id: uuidv4(),
            name: '',
            description: '',
            wikidataId: undefined,
            customProperties: {}
          };
          newItems.push(newItem);
          
          // Don't auto-save empty items immediately
          // They will be saved when user starts typing
        }
      }
      
      return newItems;
    });
  }, [saveItemMutation, queryClient, url]);

  const removeItem = useCallback((index: number) => {
    const itemId = items[index].id;
    deleteItemMutation.mutate(itemId);
    setItems(prev => prev.filter((_, i) => i !== index));
  }, [items, deleteItemMutation]);

  const removeProperty = useCallback((property: string) => {
    deletePropertyMutation.mutate(property);
    setCustomProperties(prev => prev.filter(p => p !== property));
    setSelectedProperties(prev => {
      const newSelected = { ...prev };
      delete newSelected[property];
      return newSelected;
    });
    setItems(prev => prev.map(item => ({
      ...item,
      customProperties: Object.fromEntries(
        Object.entries(item.customProperties).filter(([key]) => key !== property)
      )
    })));
  }, [deletePropertyMutation]);

    const fetchPropertyLabelsHelper = useCallback(async (propertyIds: string[]): Promise<Record<string, string>> => {
    const labels = await fetchPropertyLabels(propertyIds);
    setPropertyLabels(prev => ({ ...prev, ...labels }));
    return labels;
  }, []);

  // Modify the addProperty function to handle auto-population
  const addProperty = useCallback(async (property: string) => {
    console.log('Adding property:', property);
    const normalizedProperty = property.replace('http://www.wikidata.org/prop/', '');
    
    // Check if property is invalid,
    if (!normalizedProperty) {
      console.log('Property is invalid:', normalizedProperty);
      return;
    }

    // Check if property is already in customProperties array (visible in UI)
    if (customProperties.includes(normalizedProperty)) {
      console.log('Property already visible in UI:', normalizedProperty);
      return;
    }
    
    console.log('Proceeding to add property:', normalizedProperty);
    
    // FIRST: Add to selected properties and custom properties (optimistic update)
    setSelectedProperties(prev => ({ ...prev, [normalizedProperty]: true }));
    setCustomProperties(prev => [...prev, normalizedProperty]);
    
    // SECOND: Fetch label if not exists
    if (!propertyLabels[normalizedProperty]) {
      try {
        const labels = await fetchPropertyLabels([normalizedProperty]);
        setPropertyLabels(prev => ({ ...prev, ...labels }));
      } catch (error) {
        console.error('Error fetching property labels:', error);
      }
    }
    
    // THIRD: Add to all items with auto-population from propertyCache
    setItems(prev => prev.map(item => {
      const existingValue = item.customProperties[normalizedProperty] || '';
      let autoPopulatedValue = existingValue;
      
      // If item has Wikidata ID and we have cached properties, use the cached value
      if (item.wikidataId && propertyCache[item.wikidataId] && propertyCache[item.wikidataId][normalizedProperty]) {
        autoPopulatedValue = propertyCache[item.wikidataId][normalizedProperty];
        console.log(`Auto-populating ${normalizedProperty} for ${item.wikidataId} with value:`, autoPopulatedValue);
      }
      
      return {
        ...item,
        customProperties: {
          ...item.customProperties,
          [normalizedProperty]: autoPopulatedValue
        }
      };
    }));
    
    // FOURTH: Save to database (but don't invalidate queries immediately)
    try {
      await addPropertyToDb(url, normalizedProperty);
      console.log('Property successfully saved to database:', normalizedProperty);
    } catch (error) {
      console.error('Error saving property to database:', error);
      // Revert optimistic updates on error
      setSelectedProperties(prev => {
        const newSelected = { ...prev };
        delete newSelected[normalizedProperty];
        return newSelected;
      });
      setCustomProperties(prev => prev.filter(p => p !== normalizedProperty));
      setItems(prev => prev.map(item => ({
        ...item,
        customProperties: Object.fromEntries(
          Object.entries(item.customProperties).filter(([key]) => key !== normalizedProperty)
        )
      })));
      return;
    }
    
    // FIFTH: Fetch Wikidata properties for items with wikidata_id that don't have cached data
    const itemsWithWikidataId = items.filter(item => item.wikidataId && !propertyCache[item.wikidataId]);
    
    for (const item of itemsWithWikidataId) {
      if (item.wikidataId) {
        try {
          console.log('Fetching Wikidata properties for:', item.wikidataId);
          const properties = await fetchWikidataProperties(item.wikidataId);
          setPropertyCache(prev => ({ ...prev, [item.wikidataId!]: properties }));
          
          // Update the item with the fetched property value if it exists
          if (properties[normalizedProperty]) {
            console.log(`Updating ${item.wikidataId} with ${normalizedProperty}:`, properties[normalizedProperty]);
            setItems(prevItems => prevItems.map(prevItem => 
              prevItem.wikidataId === item.wikidataId
                ? {
                    ...prevItem,
                    customProperties: {
                      ...prevItem.customProperties,
                      [normalizedProperty]: properties[normalizedProperty]
                    }
                  }
                : prevItem
            ));
          }
        } catch (error) {
          console.error(`Error fetching Wikidata properties for ${item.wikidataId}:`, error);
        }
      }
    }
    
    console.log('Property addition completed:', normalizedProperty);
  }, [selectedProperties, propertyLabels, items, propertyCache, url]);

  const handleWikidataSelect = useCallback(async (suggestion: WikidataSuggestion, itemId: string) => {
    console.log('Wikidata selection for item:', itemId, suggestion);
    
    // Update item immediately with suggestion data
    setItems(prev => prev.map(item => 
      item.id === itemId
        ? {
            ...item,
            name: suggestion.label || '',
            description: suggestion.description || '',
            wikidataId: suggestion.id
          }
        : item
    ));

    // Save the updated item
    const updatedItem = items.find(item => item.id === itemId);
    if (updatedItem) {
      const itemToSave = {
        ...updatedItem,
        name: suggestion.label || '',
        description: suggestion.description || '',
        wikidataId: suggestion.id
      };
      saveItemMutation.mutate(itemToSave);
    }
    
    // Fetch properties for this Wikidata item
    if (suggestion.id) {
      const properties = await fetchWikidataProperties(suggestion.id);
      setPropertyCache(prev => ({ ...prev, [suggestion.id]: properties }));
      
      // Update item with fetched properties
      setItems(prev => prev.map(item => 
        item.id === itemId
          ? {
              ...item,
              customProperties: {
                ...item.customProperties,
                ...Object.fromEntries(
                  Object.entries(properties).filter(([key]) => selectedProperties[key])
                )
              }
            }
          : item
      ));
    }
  }, [selectedProperties, items, saveItemMutation]);

  const handleNewPropertyKeyDown = useCallback((e: React.KeyboardEvent<HTMLInputElement>) => {
    if (e.key === 'Enter') {
      const input = e.currentTarget;
      const inputValue = input.value.trim();
      
      if (inputValue) {
        // Extract property ID from "Label (P123)" format
        const propertyId = inputValue.includes(' (') && inputValue.endsWith(')')
          ? inputValue.split(' (').pop()?.slice(0, -1) || inputValue
          : inputValue;
        
        addProperty(propertyId);
        input.value = '';
      }
    }
  }, [addProperty]);

  if (isLoading) {
    return <div className="p-4">Loading items...</div>;
  }

  if (error) {
    return <div className="p-4 text-red-600">Error loading items: {error.message}</div>;
  }

  return (
    <div className="p-4">
      <div className="table-container">
        <table>
          <tbody>
            {/* Name row */}
            <tr>
              <td className="font-medium">Name</td>
              {items.map((item, index) => (
                <td key={`name-${item.id}`}>
                  <div className="flex items-center gap-2">
                    <div className="flex-1">
                      <TypeaheadInput
                        value={item.name}
                        onInput={(value) => updateItem(index, 'name', value)}
                        onSelect={(suggestion) => handleWikidataSelect(suggestion, item.id)}
                        fetchSuggestions={fetchWikidataSuggestions}
                        onFocus={() => setFocusedItemId(item.id)}
                        placeholder="Enter item name"
                      />
                    </div>
                    <button
                      onClick={() => removeItem(index)}
                      className="delete-button item-delete"
                      title="Delete item"
                    >
                      ×
                    </button>
                  </div>
                </td>
              ))}
            </tr>
            
            {/* Description row */}
            <tr>
              <td className="font-medium">Description</td>
              {items.map((item, index) => (
                <td key={`description-${item.id}`}>
                  <EditableCell
                    value={item.description}
                    onInput={(value) => updateItem(index, 'description', value)}
                    focusedCell={focusedCell}
                    setFocusedCell={setFocusedCell}
                    cellKey={`description-${index}`}
                    inputType="textarea"
                  />
                </td>
              ))}
            </tr>
            
            {/* Custom property rows */}
            {customProperties.map((property) => {
              const propertyLabel = propertyLabels[property] || property;
              return (
                <tr key={property}>
                  <td className="font-medium relative">
                    {propertyLabel}
                    <button
                      onClick={() => removeProperty(property)}
                      className="delete-button property-delete"
                      title="Delete property"
                    >
                      ×
                    </button>
                  </td>
                  {items.map((item, index) => (
                    <td key={`${property}-${item.id}`}>
                      <EditableCell
                        value={item.customProperties[property] || ''}
                        onInput={(value) => updateItem(index, property, value)}
                        focusedCell={focusedCell}
                        setFocusedCell={setFocusedCell}
                        cellKey={`${property}-${index}`}
                        inputType="textarea"
                      />
                    </td>
                  ))}
                </tr>
              );
            })}
          </tbody>
        </table>
      </div>
      
      {/* Add new property input */}
      <div className="mt-4">
        <PropertyInput
          propertyCache={propertyCache}
          propertyLabels={propertyLabels}
          onAddProperty={addProperty}
          onFetchPropertyLabels={fetchPropertyLabelsHelper}
          className="w-80"
        />
      </div>
    </div>
  );
}